
import ses_mailer
import flask_mail

from pylot.component import mailer, cache, storage

def test_mailer_none():
    assert mailer.mail is None

def test_mailer_ses():
    class FakeFlask(object):
        config = {
            "MAILER_PROVIDER": "SES",
            "MAILER_SES_ACCESS_KEY": "",
            "MAILER_SES_SECRET_KEY": ""
        }
    app = FakeFlask
    mailer.init_app(app)

    assert isinstance(mailer.mail, ses_mailer.Mail)

def test_mailer_smtp():
    class FakeFlask(object):
        config = {
            "MAILER_PROVIDER": "SMTP",
            "MAILER_SMTP_URI": "smtp://user:pass@mail.google.com:25",
            "DEBUG": False,
            "TESTING": False
        }
    app = FakeFlask()
    mailer.init_app(app)

    assert isinstance(mailer.mail, flask_mail.Mail)


"""
Post Admin
"""

import datetime

from pylot import (Pylot,
                   utils,
                   route,
                   abort,
                   redirect,
                   request,
                   url_for,
                   jsonify,
                   session,
                   AppError)

from pylot.component import (mailer,
                             storage)
from flask_login import (LoginManager,
                         login_required,
                         login_user,
                         logout_user,
                         current_user)


def post_admin(model, **kwargs):

    def wrapper(view):
        template_dir = kwargs["template_dir"] if "template_dir" \
                                                 in kwargs else "Pylot/PostAdmin"
        template_page = template_dir + "/%s.html"
        PostStruct = model.PostStruct

        Pylot.set_context__(COMPONENT_POST_ADMIN=True)

        @view.extends__
        class AdminPost(object):
            route_base = "post-admin"
            __session_edit_key = "admin_post_edit_post"

            @route("/", endpoint="PostAdmin:index")
            def index(self):
                """
                List all posts
                """
                self.set_meta__(title="All Posts")
                per_page = self.get_config__("PAGINATION_PER_PAGE", 25)
                page = request.args.get("page", 1)
                id = request.args.get("id", None)
                slug = request.args.get("slug", None)
                status = request.args.get("status", "all")
                user_id = request.args.get("user_id", None)
                type_id = request.args.get("type_id", None)
                category_id = request.args.get("category_id", None)

                posts = PostStruct.Post.all()

                if id:
                    posts = posts.filter(PostStruct.Post.id == id)
                if slug:
                    posts = posts.filter(PostStruct.Post.slug == slug)
                if user_id:
                    posts = posts.filter(PostStruct.Post.user_id == user_id)
                if type_id:
                    posts = posts.filter(PostStruct.Post.type_id == type_id)
                if category_id:
                    posts = posts.join(PostStruct.PostCategory)\
                        .join(PostStruct.Category)\
                        .filter(PostStruct.Category.id == category_id)
                if status == "publish":
                    posts = posts.filter(PostStruct.Post.is_published == True)
                elif status == "draft":
                    posts = posts.filter(PostStruct.Post.is_draft == True)
                elif status == "revision":
                    posts = posts.filter(PostStruct.Post.is_revision == True)

                posts = posts.order_by(PostStruct.Post.id.desc())
                posts = posts.paginate(page=page, per_page=per_page)

                return self.render(posts=posts,
                                   query_vars={
                                       "id": id,
                                       "slug": slug,
                                       "user_id": user_id,
                                       "type_id": type_id,
                                       "status": status
                                   },
                                   view_template=template_page % "index")

            @route("read/<id>", endpoint="PostAdmin:read")
            def read(self, id):
                """
                Read Post
                """
                post = PostStruct.Post.get(id)
                if not post:
                    abort(404, "Post doesn't exist")

                self.set_meta__(title="Read: %s " % post.title)

                return self.render(post=post,
                                   view_template=template_page % "read")

            @route("upload-image", methods=["POST"], endpoint="PostAdmin:upload_image")
            def upload_image(self):
                """
                Placeholder for markdown
                """

                url = ""
                if request.files.get("file"):
                    url = storage.put(request.files.get('file'))

                else:
                    return "Couldn't upload file. No file exist", 401

                return self.render(file_url=url)

                # For when there is an error
                error = False
                if error:
                    return "error message", 401

                return jsonify({
                    "id": "",
                    "url": "", # full image url
                })

            @route("new", defaults={"id": None}, endpoint="PostAdmin:new")
            @route("edit/<id>", endpoint="PostAdmin:edit")
            def edit(self, id):
                """
                Create / Edit Post
                """
                self.set_meta__(title="Edit Post")

                types = [(t.id, t.name) for t in PostStruct.Type.all().order_by(PostStruct.Type.name.asc())]
                categories = [(c.id, c.name) for c in PostStruct.Category.all().order_by(PostStruct.Category.name.asc())]
                checked_cats = []
                post = {
                    "id": 0,
                    "title": "",
                    "content": "",
                    "slug": "",
                    "type_id": 0
                }

                # saved in session
                if request.args.get("error") and self.__session_edit_key in session:
                    post = session[self.__session_edit_key]
                    checked_cats = post["post_categories"]
                    del session[self.__session_edit_key]

                elif id:
                    post = PostStruct.Post.get(id)
                    if not post or post.is_revision:
                        abort(404, "Post doesn't exist")
                    checked_cats = [c.id for c in post.categories]

                return self.render(post=post,
                                   types=types,
                                   categories=categories,
                                   checked_categories=checked_cats,
                                   view_template=template_page % "edit"
                                  )

            @route("post", methods=["POST"], endpoint="PostAdmin:post")
            def post(self):
                id = request.form.get("id", None)
                title = request.form.get("title", None)
                slug = request.form.get("slug", None)
                content = request.form.get("content", None)
                type_id = request.form.get("type_id", None)
                post_categories = request.form.getlist("post_categories")
                published_date = request.form.get("published_date", None)
                status = request.form.get("status", "draft")
                is_published = True if status == "publish" else False
                is_draft = True if status == "draft" else False
                is_public = request.form.get("is_public", True)

                if status in ["draft", "publish"] and (not title or not type_id):
                    if not title:
                        self.flash_error__("Post Title is missing ")
                    if not type_id:
                        self.flash_error__("Post type is missing")

                    session[self.__session_edit_key] = {
                        "id": id,
                        "title": title,
                        "content": content,
                        "slug": slug,
                        "type_id": type_id,
                        "published_date": published_date,
                        "post_categories": post_categories
                    }
                    if id:
                        url = url_for("PostAdmin:edit", id=id, error=1)
                    else:
                        url = url_for("PostAdmin:new", error=1)
                    return redirect(url)

                data = {
                    "title": title,
                    "content": content,
                    "type_id": type_id
                }

                if published_date:
                    published_date = datetime.datetime.strptime(published_date, "%Y-%m-%d %H:%M:%S")
                else:
                    published_date = datetime.datetime.now()

                if id and status in ["delete", "revision"]:
                    post = PostStruct.Post.get(id)
                    if not post:
                        abort(404, "Post '%s' doesn't exist" % id)

                    if status == "delete":
                        post.delete()
                        self.flash_success__("Post deleted successfully!")
                        return redirect(url_for("%s:index" % view ))

                    elif status == "revision":
                        data.update({
                            "user_id": current_user.id,
                            "parent_id": id,
                            "is_revision": True,
                            "is_draft": False,
                            "is_published": False,
                            "is_public": False
                        })
                        post = PostStruct.Post.create(**data)
                        return jsonify({"revision_id": post.id})

                elif status in ["draft", "publish"]:
                    data.update({
                        "is_published": is_published,
                        "is_draft": is_draft,
                        "is_revision": False,
                        "is_public": is_public
                    })
                    if id:
                        post = PostStruct.Post.get(id)
                        if not post:
                            abort(404, "Post '%s' doesn't exist" % id)
                        elif post.is_revision:
                            abort(403, "Can't access this post")
                        else:
                            post.update(**data)
                    else:
                        data["user_id"] = current_user.id
                        if is_published:
                            data["published_date"] = published_date
                        post = PostStruct.Post.create(**data)

                    post.set_slug(slug or title)

                    post.update_categories(map(int, post_categories))

                    if post.is_published and not post.published_date:
                        post.update(published_date=published_date)

                    self.flash_success__("Post saved successfully!")

                    endpoint = "read" if post.is_published else "edit"
                    return redirect(url_for("%s:%s" % (view, endpoint), id=post.id))

                else:
                    abort(400, "Invalid post status")

            @route("categories", methods=["GET", "POST"], endpoint="PostAdmin:categories")
            def categories(self):
                self.set_meta__(title__prepend="Post Categories")
                if request.method == "POST":
                    id = request.form.get("id", None)
                    action = request.form.get("action")
                    name = request.form.get("name")
                    slug = request.form.get("slug", None)
                    ajax = request.form.get("ajax", False)
                    try:
                        if not id:
                            cat = PostStruct.Category.new(name=name, slug=slug)
                            if ajax:
                                return jsonify({
                                    "id": cat.id,
                                    "name": cat.name,
                                    "slug": cat.slug,
                                    "status": "OK"
                                })
                            self.flash_success__("New category '%s' added" % name)
                        else:
                            post_cat = PostStruct.Category.get(id)
                            if post_cat:
                                if action == "delete":
                                    post_cat.delete()
                                    self.flash_success__("Category '%s' deleted successfully!" % post_cat.name)
                                else:
                                    post_cat.update(name=name, slug=slug)
                                    self.flash_success__("Category '%s' updated successfully!" % post_cat.name)
                    except Exception as ex:
                        if ajax:
                            return jsonify({
                                "error": True,
                                "error_message": ex.message
                            })

                        self.flash_error__("Error: %s" % ex.message)
                    return redirect(url_for("%s:categories" % view))

                else:
                    cats = PostStruct.Category.all().order_by(PostStruct.Category.name.asc())
                    return self.render(categories=cats,
                                       view_template=template_page % "categories")

            @route("types", methods=["GET", "POST"], endpoint="PostAdmin:types")
            def types(self):
                self.set_meta__(title__prepend="Post Types")
                if request.method == "POST":
                    try:
                        id = request.form.get("id", None)
                        action = request.form.get("action")
                        name = request.form.get("name")
                        slug = request.form.get("slug", None)
                        if not id:
                            PostStruct.Type.new(name=name, slug=slug)
                            self.flash_success__("New type '%s' added" % name)
                        else:
                            post_type = PostStruct.Type.get(id)
                            if post_type:
                                if action == "delete":
                                    post_type.delete()
                                    self.flash_success__("Type '%s' deleted successfully!" % post_type.name)
                                else:
                                    post_type.update(name=name, slug=slug)
                                    self.flash_success__("Type '%s' updated successfully!" % post_type.name)
                    except Exception as ex:
                        self.flash_error__("Error: %s" % ex.message)
                    return redirect(url_for("%s:types" % view))
                else:
                    types = PostStruct.Type.all().order_by(PostStruct.Type.name.asc())
                    return self.render(types=types,
                                       view_template=template_page % "types")

        return view
    return wrapper


"""
Pylot command line tool

manage.py

Command line tool to manage your application

"""

import argparse
from application import get_config
import application.model as model
from pylot import utils

config = get_config()
NAME = "Pylot Manager"
__version__ = config.APP_VERSION


def setup():

    # Create all db
    model.db.create_all()

    # :: USERS
    # SETUP USER ROLES
    roles = [
                {
                    "name": "SUPERADMIN",
                    "level": 99
                },
                {
                    "name": "ADMIN",
                    "level": 98
                },
                {
                    "name": "USER",
                    "level": 1
                },
             ]
    [model.UserStruct.Role.new(name=role["name"], level=role["level"]) for role in roles]

    # ADD SUPER ADMIN
    email = config.ADMIN_EMAIL
    name = config.ADMIN_NAME
    if utils.is_valid_email(email):
        user = model.UserStruct.User.get_by_email(email)
        if not user:
            model.UserStruct.User.new(email=email, name=name, role="superadmin")
    else:
        raise AttributeError("Couldn't create new SUPERADMIN. 'email' is invalid")


    # :: POSTS
    # Set types
    post_types = ["Blog", "Article", "Page", "Other"]
    if not model.PostStruct.Type.all().count():
        [model.PostStruct.Type.new(t) for t in post_types]

    # Set categories
    post_categories = ["Blog"]
    if not model.PostStruct.Category.all().count():
        [model.PostStruct.Category.new(c) for c in post_categories]

    posts = [
        {
            "title": "About Us",
            "slug": "about",
            "type": "Page"
        },
        {
            "title": "Terms of Service",
            "slug": "tos",
            "type": "Page"
        }
    ]

def main():
    parser = argparse.ArgumentParser(description="%s  v.%s" % (NAME, __version__))
    parser.add_argument("--setup", help="Setup the system",  action="store_true")
    parser.add_argument("--upload-static-to-s3", help="Upload all static files to S3", action="store_true")
    arg = parser.parse_args()

    if arg.setup:
        # Default setup
        print("Setting up...")
        setup()

    if arg.upload_static_to_s3:
        # Upload static files to s3
        import flask_s3
        import run_www  # Or the main application run file
        print("Upload static files to S3")
        flask_s3.create_all(run_www.app)

if __name__ == "__main__":
    main()





"""
User Account
"""

import os
import datetime
from hashlib import sha1
import time
import base64
import hmac
import urllib

from pylot import (Pylot,
                   utils,
                   route,
                   abort,
                   redirect,
                   request,
                   url_for,
                   jsonify,
                   session,
                   AppError)

from pylot.component import (mailer,
                             storage,
                             recaptcha)
from flask_login import (LoginManager,
                         login_required,
                         login_user,
                         logout_user,
                         current_user)

def user_account(model, **kwargs):
    """
    This view is extendable

    kwargs:
        - on_signin_view
        - on_signout_view
        - template_dir

    """
    def wrapper(view):

        Pylot.set_context__(COMPONENT_LOGIN=True)
        view_name = view.__name__
        User = model.UserStruct.User

        # Login
        login_view = "%s:login" % view_name
        on_signin_view = kwargs["on_signin_view"] if "on_signin_view" \
                                                     in kwargs else "Index:index"
        on_signout_view = kwargs["on_signout_view"] if "on_signout_view" \
                                                       in kwargs else "Index:index"
        template_dir = kwargs["template_dir"] if "template_dir" \
                                                 in kwargs else "Pylot/UserAccount"
        template_page = template_dir + "/%s.html"

        login_manager = LoginManager()
        login_manager.login_view = login_view
        login_manager.login_message_category = "error"
        Pylot.bind(login_manager.init_app)

        @login_manager.user_loader
        def load_user(userid):
            return User.get(userid)

        @view.extends__
        class Login(object):

            SESSION_KEY_SET_EMAIL_DATA = "set_email_tmp_data"

            def _can_login(self):
                if not self.get_config__("LOGIN_EMAIL_ENABLE"):
                    abort(403)

            def _can_oauth_login(self):
                if not self.get_config__("LOGIN_OAUTH_ENABLE"):
                    abort(403)

            def _can_signup(self):
                if not self.get_config__("LOGIN_SIGNUP_ENABLE"):
                    abort(403)

            def _login_user(self, user_context):
                login_user(user_context)
                user_context.update_last_login()
                user_context.update_last_visited()

            @classmethod
            def signup_handler(cls):
                """
                To handle the signup process. Must still bind to the app
                 :returns User object:
                """
                if request.method == "POST":
                    name = request.form.get("name")
                    email = request.form.get("email")
                    password = request.form.get("password")
                    password2 = request.form.get("password2")
                    profile_pic_url = request.form.get("profile_pic_url", None)

                    if not name:
                        raise UserWarning("Name is required")
                    elif not utils.is_valid_email(email):
                        raise UserWarning("Invalid email address '%s'" % email)
                    elif not password.strip() or password.strip() != password2.strip():
                        raise UserWarning("Passwords don't match")
                    elif not utils.is_valid_password(password):
                        raise UserWarning("Invalid password")
                    else:
                        return User.new(email=email,
                                        password=password.strip(),
                                        name=name,
                                        profile_pic_url=profile_pic_url,
                                        signup_method="EMAIL")

            @classmethod
            def change_login_handler(cls, user_context=None, email=None):
                if not user_context:
                    user_context = current_user
                if not email:
                    email = request.form.get("email").strip()

                if not utils.is_valid_email(email):
                    raise UserWarning("Invalid email address '%s'" % email)
                else:
                    if email != user_context.email and User.get_by_email(email):
                        raise UserWarning("Email exists already '%s'" % email)
                    elif email != user_context.email:
                        user_context.update(email=email)
                        return True
                return False

            @classmethod
            def change_password_handler(cls, user_context=None, password=None,
                                        password2=None):
                if not user_context:
                    user_context = current_user
                if not password:
                    password = request.form.get("password").strip()
                if not password2:
                    password2 = request.form.get("password2").strip()

                if password:
                    if password != password2:
                        raise UserWarning("Password don't match")
                    elif not utils.is_valid_password(password):
                        raise UserWarning("Invalid password")
                    else:
                        user_context.set_password(password)
                        return True
                else:
                    raise UserWarning("Password is empty")

            @classmethod
            def reset_password_handler(cls, user_context=None,
                                       delivery=None,
                                       send_notification=True):
                """
                Reset the password
                :returns string: The new password string
                """
                if not user_context:
                    user_context = current_user

                user = user_context
                method_ = cls.get_config__("LOGIN_RESET_PASSWORD_METHOD", "").upper()
                new_password = None
                if method_ == "TOKEN":
                    token = user.set_reset_password_token()
                    url = url_for("UserAccount:reset_password_token",
                                  token=token,
                                  _external=True)
                else:
                    new_password = user.set_random_password()
                    url = url_for("UserAccount:login", _external=True)

                mailer.send_template("reset-password.txt",
                                     method_=method_,
                                     to=user.email,
                                     name=user.email,
                                     url=url,
                                     new_password=new_password)

            @route("sign_s3_upload", endpoint="UserAccount:sign_s3_upload")
            def sign_s3_upload(self):
                """
                Allow to create Signed object to upload to S3 via JS
                """
                AWS_ACCESS_KEY = self.get_config__('AWS_ACCESS_KEY_ID')
                AWS_SECRET_KEY = self.get_config__('AWS_SECRET_ACCESS_KEY')
                S3_BUCKET = self.get_config__('AWS_S3_BUCKET_NAME')

                object_name = request.args.get('s3_object_name')
                mime_type = request.args.get('s3_object_type')
                expires = long(time.time()+10)
                amz_headers = "x-amz-acl:public-read"
                put_request = "PUT\n\n%s\n%d\n%s\n/%s/%s" % (mime_type, expires, amz_headers, S3_BUCKET, object_name)
                signature = base64.encodestring(hmac.new(AWS_SECRET_KEY, put_request, sha1).digest())
                signature = urllib.quote(urllib.quote_plus(signature.strip()))
                url = 'https://s3.amazonaws.com/%s/%s' % (S3_BUCKET, object_name)
                return jsonify({
                    'signed_request': '%s?AWSAccessKeyId=%s&Expires=%d&Signature=%s' % (url, AWS_ACCESS_KEY, expires, signature),
                     'url': url
                  })

            @route("login", endpoint="UserAccount:login")
            def login(self):
                """ Login page """
                self._can_login()
                logout_user()
                self.set_meta__(title="Login")
                return self.render(login_url_next=request.args.get("next", ""),
                                   view_template=template_page % "login")

            @route("logout", endpoint="UserAccount:logout")
            def logout(self):
                logout_user()
                self.flash_success__("Logout successfully!")
                return redirect(url_for(on_signout_view or login_view))

            @route("signup", methods=["GET", "POST"], endpoint="UserAccount:signup")
            def signup(self):
                self._can_login()
                self._can_signup()
                self.set_meta__(title="Signup")
                if request.method == "POST":
                    # reCaptcha
                    if not recaptcha.verify():
                        self.flash_error__("Invalid Security code")
                        return redirect(url_for("UserAccount:signup", next=request.form.get("next")))
                    try:
                        new_account = self.signup_handler()
                        login_user(new_account)
                        self.flash_success__("Congratulations! ")
                        return redirect(request.form.get("next") or url_for(on_signin_view))
                    except Exception as ex:
                        self.flash_error__(ex.message)
                    return redirect(url_for("%s:signup" % view_name, next=request.form.get("next")))

                logout_user()
                return self.render(login_url_next=request.args.get("next", ""),
                                   view_template=template_page % "signup")

            @route("lost-password", methods=["GET", "POST"], endpoint="UserAccount:lost_password")
            def lost_password(self):
                self._can_login()
                self.set_meta__(title="Lost Password")
                if request.method == "POST":
                    email = request.form.get("email")
                    user = User.get_by_email(email)
                    if user:
                        delivery = self.get_config__("LOGIN_RESET_PASSWORD_DELIVERY")
                        self.reset_password_handler(user_context=user, delivery=delivery)
                        self.flash_success__("A new password has been sent to '%s'" % email)
                    else:
                        self.flash_error__("Invalid email address")
                    return redirect(url_for(login_view))
                else:
                    logout_user()
                    return self.render(view_template=template_page % "lost_password")

            @route("email-login", methods=["POST"], endpoint="UserAccount:email_login")
            def email_login(self):
                """ login via email """
                self._can_login()

                email = request.form.get("email").strip()
                password = request.form.get("password").strip()

                if not email or not password:
                    self.flash_error__("Email or Password is empty")
                    return redirect(url_for(login_view, next=request.form.get("next")))
                account = User.get_by_email(email)
                if account and account.password_matched(password):
                    self._login_user(account)
                    return redirect(request.form.get("next") or url_for(on_signin_view))
                else:
                    self.flash_error__("Email or Password is invalid")
                    return redirect(url_for(login_view, next=request.form.get("next")))

            # OAUTH Login
            @route("oauth-login", methods=["POST"], endpoint="UserAccount:oauth_login")
            def oauth_login(self):
                """ To login via social """
                self._can_oauth_login()

                email = request.form.get("email").strip()
                name = request.form.get("name").strip()
                provider = request.form.get("provider").strip()
                provider_user_id = request.form.get("provider_user_id").strip()
                image_url = request.form.get("image_url").strip()
                next = request.form.get("next", "")
                # save to session and redirect to enter email address
                if not email:
                    session[self.SESSION_KEY_SET_EMAIL_DATA] = {
                        "type": "social_login",
                        "email": email,
                        "name": name,
                        "provider": provider,
                        "provider_user_id": provider_user_id,
                        "image_url": image_url,
                        "next": next,
                        "signup_method": "SOCIAL:%s" % provider.upper()
                    }
                    return redirect(url_for("UserAccount:set_email", next=request.form.get("next", "")))
                else:
                    user = User.oauth_register(provider=provider,
                                               provider_user_id=provider_user_id,
                                               email=email,
                                               name=name,
                                               image_url=image_url,
                                               signup_method="SOCIAL:%s" % provider.upper())
                    if user:
                        self._login_user(user)
                        return redirect(request.form.get("next") or url_for(on_signin_view))

                return redirect(url_for(login_view, next=request.form.get("next", "")))

            @route("reset-password-token/<token>", endpoint="UserAccount:reset_password_token")
            def reset_password_token(self, token):
                self._can_login()
                user = User.get_by_token(token)
                if not user:
                    self.flash_error__("Invalid reset password token. Please try again")
                    return redirect(url_for("%s:lost_password" % view_name))
                else:
                    self._login_user(user)
                    return redirect(url_for("%s:reset_password" % view_name))

            @route("reset-password", methods=["GET", "POST"], endpoint="UserAccount:reset_password")
            @login_required
            def reset_password(self):
                self._can_login()
                self.set_meta__(title="Reset Password")
                if current_user.require_password_change:
                    if request.method == "POST":
                        try:
                            self.change_password_handler()
                            current_user.clear_reset_password_token()
                            self.flash_success__("Password updated successfully!")
                            return redirect(url_for(on_signin_view))
                        except Exception as ex:
                            self.flash_error__("Error: %s" % ex.message)
                            return redirect(url_for("%s:reset_password" % view_name))
                    return self.render(view_template=template_page % "reset_password")
                return redirect(url_for(on_signin_view))

            @route("set-email", methods=["GET", "POST"], endpoint="UserAccount:set_email")
            @login_required
            def set_email(self):
                self._can_login()
                self.set_meta__(title="Set Email")

                # Only user without email can set email
                if current_user.email:
                    return redirect(url_for("%s:account_settings" % view_name))

                if request.method == "POST":
                    email = request.form.get("email")
                    if not utils.is_valid_email(email):
                        self.flash_error__("Invalid email address '%s'" % email)
                        return redirect(url_for(login_view))

                    if email and self.SESSION_KEY_SET_EMAIL_DATA in session:
                        _data = session[self.SESSION_KEY_SET_EMAIL_DATA]
                        user = User.get_by_email(email)
                        if user:
                            self.flash_error__("An account is already using '%s'" % email)
                        else:
                            User.new(email=email,
                                     name=_data["name"],
                                     signup_method=_data["signup_method"] if "signup_method" in _data else "" )

                            if "type" in _data:
                                if _data["type"] == "social_login":
                                    user = User.social_login(provider=_data["provider"],
                                                             provider_user_id=_data["provider_user_id"],
                                                             email=email,
                                                             name=_data["name"],
                                                             image_url=_data["image_url"])
                                    return redirect(request.form.get("next") or url_for(on_signin_view))

                        return redirect(url_for("%s:set_email" % view_name,
                                                next=request.form.get("next", "")))
                else:
                    return self.render(view_template=template_page % "set_email")

            @route("account-settings", endpoint="UserAccount:account_settings")
            @login_required
            def account_settings(self):
                self.set_meta__(title="Account Settings")
                return self.render(view_template=template_page % "account_settings")

            @route("oauth-connect", methods=["POST"], endpoint="UserAccount:oauth_connect")
            @login_required
            def oauth_connect(self):
                """ To login via social """
                email = request.form.get("email").strip()
                name = request.form.get("name").strip()
                provider = request.form.get("provider").strip()
                provider_user_id = request.form.get("provider_user_id").strip()
                image_url = request.form.get("image_url").strip()
                next = request.form.get("next", "")
                try:
                    current_user.oauth_connect(provider=provider,
                                             provider_user_id=provider_user_id,
                                             email=email,
                                             name=name,
                                             image_url=image_url)
                except Exception as ex:
                    self.flash_error__("Unable to link your account")

                return redirect(url_for("%s:account_settings" % view_name))

            @route("change-login", methods=["POST"], endpoint="UserAccount:change_login")
            @login_required
            def change_login(self):
                confirm_password = request.form.get("confirm-password").strip()
                try:
                    if current_user.password_matched(confirm_password):
                        self.change_login_handler()
                        self.flash_success__("Login Info updated successfully!")
                    else:
                        self.flash_error__("Invalid password")
                except Exception as ex:
                    self.flash_error__("Error: %s" % ex.message)
                return redirect(url_for("UserAccount:account_settings"))

            @route("change-password", methods=["POST"], endpoint="UserAccount:change_password")
            @login_required
            def change_password(self):
                try:
                    confirm_password = request.form.get("confirm-password").strip()
                    if current_user.password_matched(confirm_password):
                        self.change_password_handler()
                        self.flash_success__("Password updated successfully!")
                    else:
                        self.flash_error__("Invalid password")
                except Exception as ex:
                    self.flash_error__("Error: %s" % ex.message)
                return redirect(url_for("UserAccount:account_settings"))

            @route("change-info", methods=["POST"], endpoint="UserAccount:change_info")
            @login_required
            def change_info(self):
                name = request.form.get("name").strip()
                profile_pic_url = request.form.get("profile_pic_url", "").strip()

                data = {}
                if name and name != current_user.name:
                    data.update({"name": name})
                if profile_pic_url:
                    data.update({"profile_pic_url": profile_pic_url})
                if data:
                    current_user.update(**data)
                    self.flash_success__("Account info updated successfully!")
                return redirect(url_for("UserAccount:account_settings"))

            @route("change-profile-pic", methods=["POST"], endpoint="UserAccount:change_profile_pic")
            @login_required
            def change_profile_pic(self):
                profile_pic_url = request.form.get("profile_pic_url").strip()
                _ajax = request.form.get("_ajax", None)
                if profile_pic_url:
                    current_user.update(profile_pic_url=profile_pic_url)
                if _ajax:
                    return jsonify({})
                return redirect(url_for("UserAccount:account_settings"))

        return view
    return wrapper


"""
Pylot
"""

from pylot import (Pylot,
                   route,
                   abort,
                   request,
                   redirect,
                   url_for,
                   jsonify,
                   session)

from pylot.component import (storage,
                             mailer,
                             cache,
                             recaptcha)

from pylot.component import views 

from application import model

import datetime

@views.user_account(model=model)
class Login(Pylot):
    route_base = "/"

@views.user_admin(model=model)
class UserAdmin(Pylot):
    pass

@views.post_admin(model=model)
class PostManager(Pylot):
    pass

@views.post_reader(model=model, types=["page"])
@views.contact_page()
class Index(Pylot):
    route_base = "/"

    def index(self):
        self.set_meta__(title="Hello Pylot!")
        return self.render()

    @route("upload", methods=["GET", "POST"])
    def upload(self):
        url = ""
        if request.method == "POST":
            _file = request.files.get('file')
            resp = storage.upload(_file, prefix="my-pictures/pat2", acl="public-read")
            print resp
            return jsonify({"cdn_url": resp.get_url(),
                           "name": resp.name,
                            "size": resp.size})
        return self.render(file_url=url)


@views.post_reader(model=model, types=["blog", "article", "page"])
class Blog(Pylot):
    pass

# Example
class Example(Pylot):

    @cache.cached(10)
    def time(self):
        return str(datetime.datetime.now())

    def sendmail(self):
        mailer.send(to="mcx2082@gmail.com", subject="Pylot is looking good",
                    body="Di yo sa\n Nou sonn'on lot jan!")

        return "email sent"
    def index(self):
        self.set_meta__(title="Example Page")
        self.flash_error__("This is an error message set by error_ and called with show_flashed_message()")
        self.flash_success__("This is a success message set by error_ and called with show_flashed_message()")
        return self.render()



"""
Pylot

Command line tool

pylot -c project_name

"""

import os
import argparse
import pkg_resources
import pylot

PACKAGE = pylot
CWD = os.getcwd()
SKELETON_DIR = "app_templates"
APPLICATION_DIR = "%s/application" % CWD

def get_project_dir_path(project_name):
    return "%s/%s" % (APPLICATION_DIR, project_name)

def copy_resource(src, dest):
    """
    To copy package data to destination
    """
    dest = (dest + "/" + os.path.basename(src)).rstrip("/")
    if pkg_resources.resource_isdir(PACKAGE.__name__, src):
        if not os.path.isdir(dest):
            os.makedirs(dest)
        for res in pkg_resources.resource_listdir(__name__, src):
            copy_resource(src + "/" + res, dest)
    else:
        if os.path.splitext(src)[1] not in [".pyc"]:
            with open(dest, "wb") as f:
                f.write(pkg_resources.resource_string(__name__, src))


def create_project(project_name, template="default"):
    """
    Create the project
    """

    project_dir = get_project_dir_path(project_name)

    run_tpl = pkg_resources.resource_string(__name__, '%s/run.py' % (SKELETON_DIR))
    init_py_tpl = pkg_resources.resource_string(__name__, '%s/init.py' % (SKELETON_DIR))
    deployapp_tpl = pkg_resources.resource_string(__name__, '%s/deployapp.yml' % (SKELETON_DIR))
    config_tpl = pkg_resources.resource_string(__name__, '%s/config.py' % (SKELETON_DIR))
    model_tpl = pkg_resources.resource_string(__name__, '%s/model.py' % (SKELETON_DIR))
    manage_tpl = pkg_resources.resource_string(__name__, '%s/manage.py' % (SKELETON_DIR))

    run_file = "%s/run_%s.py" % (CWD, project_name)
    requirements_txt = "%s/requirements.txt" % CWD
    deployapp_yml = "%s/deployapp.yml" % CWD
    config_py = "%s/config.py" % APPLICATION_DIR
    model_py = "%s/model.py" % APPLICATION_DIR
    manage_py = "%s/manage.py" % CWD

    if not os.path.isdir(APPLICATION_DIR):
        os.makedirs(APPLICATION_DIR)

    if not os.path.isdir(project_dir):
        os.makedirs(project_dir)

    _app_init_py = "%s/__init__.py" % APPLICATION_DIR
    if not os.path.isfile(_app_init_py):
        with open(_app_init_py, "wb") as f:
            f.write(init_py_tpl)

    _init_py = "%s/__init__.py" % CWD
    if not os.path.isfile(_init_py):
        with open(_init_py, "wb") as f:
            f.write("# Pylot")

    if not os.path.isfile(config_py):
        with open(config_py, "wb") as f:
            f.write(config_tpl)

    if not os.path.isfile(model_py):
        with open(model_py, "wb") as f:
            f.write(model_tpl)

    if not os.path.isfile(run_file):
        with open(run_file, "wb") as f:
            f.write(run_tpl.format(project_name=project_name))

    if not os.path.isfile(requirements_txt):
        with open(requirements_txt, "wb") as f:
            f.write("%s==%s" % (PACKAGE.NAME, PACKAGE.__version__))

    if not os.path.isfile(deployapp_yml):
        with open(deployapp_yml, "wb") as f:
            f.write(deployapp_tpl)

    if not os.path.isfile(manage_py):
        with open(manage_py, "wb") as f:
            f.write(manage_tpl)

    copy_resource("%s/%s/" % (SKELETON_DIR, template), project_dir)


def main():

    _description = "%s %s" % (PACKAGE.NAME, PACKAGE.__version__)
    parser = argparse.ArgumentParser(description=_description)
    parser.add_argument("-c", "--create", help="To create a new project."
                             " [ie: pylot -c www]")

    arg = parser.parse_args()
    print(_description)
    if arg.create:
        project_name = arg.create
        template = "default"
        create_project(project_name, template)
        print("Created project: %s'" % project_name)
        print("To launch server run 'run_%s.py'" % project_name)







"""
::Pylot::

run_www.py

To run the server
"""

from pylot import Pylot

# Import get_env for the system environment to retrieve the environment
from application import get_env

# Import the application's views
import application.www.views

# The directory containing your views/static/templates
app_dir = "application/www"

# Get the environment: Development, Production. To help with config
app_env = get_env()

# The project config object
app_config = "application.config.%s" % app_env

# Pylot.init returns Flask instance
app = Pylot.init(__name__, directory=app_dir, config=app_config)

if __name__ == "__main__":
    app.run()



"""
Pylot

model.py

You may place your models here.
"""

from active_sqlalchemy import SQLAlchemy
import pylot.component.model
from . import get_config

config = get_config()

db = SQLAlchemy(config.DATABASE_URI)

# User Struct
UserStruct = pylot.component.model.user_struct(db)

# Post Struct
PostStruct = pylot.component.model.post_struct(UserStruct)

# CloudStorageStruct
CloudStorageStruct = pylot.component.model.cloud_storage_struct(db)


# A simple Note table
class MyNote(db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey(UserStruct.User.id))
    title = db.Column(db.String(250))
    content = db.Column(db.Text)
    user = db.relationship(UserStruct.User, backref="notes")



"""
Pylot command line tool

manage.py

Command line tool to manage your application

"""

import argparse
from application import get_config
import application.model as model
from pylot import utils

config = get_config()
NAME = "Pylot Manager"
__version__ = config.APP_VERSION

def setup():

    # Create all db
    model.db.create_all()

    roles = ["user", "admin", "superadmin"]

    # Setup the SUPERADMIN
    email = config.ADMIN_EMAIL
    name = config.ADMIN_NAME
    user = model.User.get_by_email(email)
    if not user:
        model.User.User.new(email=email,
                            name=name,
                            role="SUPERADMIN")


def main():
    parser = argparse.ArgumentParser(description="%s  v.%s" % (NAME, __version__))
    parser.add_argument("--setup", help="Setup the system",  action="store_true")
    parser.add_argument("--upload-static-to-s3", help="Upload all static files to S3", action="store_true")
    arg = parser.parse_args()

    if arg.setup:
        # Default setup
        print("Setting up...")
        setup()

    if arg.upload_static_to_s3:
        # Upload static files to s3
        import flask_s3
        import run_www  # Or the main application run file
        print("Upload static files to S3")
        flask_s3.create_all(run_www.app)

if __name__ == "__main__":
    main()





"""
::PYLOT::

Base config file

"""
import os


CWD = os.path.dirname(__file__)

class Config(object):
    """
    Flask config: http://flask.pocoo.org/docs/0.10/config/
    """
    SERVER_NAME = None
    DEBUG = True
    SECRET_KEY = "PLEASE CHANGE ME"

    # ------

    APP_NAME = ""
    APP_VERSION = "0.0.1"

    #
    ADMIN_EMAIL = "admin@test.com"
    ADMIN_NAME = "Admin Test"

# ------- AWS CREDENTIALS ------------------------------------------------------

    #: AWS Credentials
    # For: S3, SES Mailer, flask s3
    AWS_ACCESS_KEY_ID = ""
    AWS_SECRET_ACCESS_KEY = ""
    AWS_S3_BUCKET_NAME = ""

# ------- DATABASES ------------------------------------------------------------

    #: SQLAlchemy
    #: format: engine://USERNAME:PASSWORD@HOST:PORT/DB_NAME
    DATABASE_URI = ""

    #: REDIS
    #: format: USERNAME:PASSWORD@HOST:PORT
    REDIS_URI = None

# ------------------------------------------------------------------------------
    # WEBASSETS

    # Flask-Assets
    # http://flask-assets.readthedocs.org/
    ASSETS_DEBUG = False
    FLASK_ASSETS_USE_S3 = False

    # Flask-S3
    # https://flask-s3.readthedocs.org/en/v0.1.4/
    USE_S3 = False
    S3_BUCKET_DOMAIN = ""
    S3_BUCKET_NAME = AWS_S3_BUCKET_NAME
    S3_USE_HTTPS = False
    USE_S3_DEBUG = False
    S3_ONLY_MODIFIED = False

# ------------------------------------------------------------------------------

    #: SESSION
    #: Flask-KVSession is used to save the user's session
    #: Set the SESSION_URI to by using these examples below to set KVSession
    #: To use local session, just set SESSION_URI to None
    #:
    #: Redis: redis://username:password@host:6379/db
    #: S3: s3://username:password@s3.aws.amazon.com/bucket
    #: Google Storage: google_storage:username:password@cloud.google.com/bucket
    #: SQL: postgresql://username:password@host:3306/db
    #:      mysql+pysql://username:password@host:3306/db
    #:      sqlite://
    #: Memcached: memcache://host:port
    #:
    SESSION_URI = None

# ------------------------------------------------------------------------------

    #: CLOUDSTORAGE
    #: Flask-CloudStorage is used to save upload on S3, Google Storage,
    #: Cloudfiles, Azure Blobs, and Local storage
    #: When using local storage, they can be accessed via http://yoursite/files

    #: CLOUDSTORAGE_PROVIDER:
    # The provider to use. By default it's 'LOCAL'.
    # You can use:
    # LOCAL, S3, GOOGLE_STORAGE, AZURE_BLOBS, CLOUDFILES
    CLOUDSTORAGE_PROVIDER = "LOCAL"

    #: CLOUDSTORAGE_KEY
    # The storage key. Leave it blank if PROVIDER is LOCAL
    CLOUDSTORAGE_KEY = AWS_ACCESS_KEY_ID

    #: CLOUDSTORAGE_SECRET
    #: The storage secret key. Leave it blank if PROVIDER is LOCAL
    CLOUDSTORAGE_SECRET = AWS_SECRET_ACCESS_KEY

    #: CLOUDSTORAGE_CONTAINER
    #: The Bucket name (for S3, Google storage, Azure, cloudfile)
    #: or the directory name (LOCAL) to access
    CLOUDSTORAGE_CONTAINER = "uploads"

    #: CLOUDSTORAGE_ALLOWED_EXTENSIONS
    #: List of extensions to allow
    CLOUDSTORAGE_ALLOWED_EXTENSIONS = []

    #: CLOUDSTORAGE_LOCAL_PATH
    #: When POVIDER is LOCAL, the directory path where CONTAINER exists
    CLOUDSTORAGE_LOCAL_PATH = "%s/var" % CWD

    #: CLOUDSTORAGE_LOCAL_URL
    #: Url to access LOCAL file
    CLOUDSTORAGE_SERVE_FILES = True

    CLOUDSTORAGE_SERVE_FILES_URL = "files"

    #: CLOUDSTORAGE_SERVE_FILES_URL_SECURE
    #: Bool to serve files via http or https
    CLOUDSTORAGE_SERVE_FILES_URL_SECURE = False

    CLOUDSTORAGE_URI = "s3://{username}:{password}@s3.aws.amazon.com/{bucket}"\
        .format(username=AWS_SECRET_ACCESS_KEY,
                password=AWS_SECRET_ACCESS_KEY,
                bucket=AWS_S3_BUCKET_NAME)

# ------------------------------------------------------------------------------

    #: MAILER
    #: To send mail using AWS SES or SMTP
    #: You can send raw email, or templated email for convenience

    #: MAILER_PROVIDER
    #: The mailer provider SES or SMTP
    MAILER_PROVIDER = "SMTP"

    #: MAILER_SES_ACCESS_KEY
    #: For SES The AWS_ACCESS_KEY_ID
    MAILER_SES_ACCESS_KEY = AWS_ACCESS_KEY_ID

    #: MAILER_SES_SECRET_KEY
    #: For SES The AWS_SECRET_ACCESS_KEY
    MAILER_SES_SECRET_KEY = AWS_SECRET_ACCESS_KEY

    #: MAILER_SENDER - The sender of the email by default
    #: For SES, this email must be authorized
    MAILER_SENDER = ""

    #: MAILER_REPLY_TO
    #: The email to reply to by default
    MAILER_REPLY_TO = ""

    #: MAILER_TEMPLATE
    #: a directory that contains the email template or a dict
    MAILER_TEMPLATE = "%s/var/ses-mailer" % CWD

    #: MAILER_TEMPLATE_CONTEXT
    #: a dict of all context to pass to the email by default
    MAILER_TEMPLATE_CONTEXT = {
        "site_name": "MyTestSite.com",
        "site_url": "http://mytestsite.com"
    }

    #: MAILER_SMTP_URI
    #: The uri for the smtp connection. It will use Flask-Mail
    #: format: smtp://USERNAME:PASSWORD@HOST:PORT
    #: with sll -> smtp+ssl://USERNAME:PASSWORD@HOST:PORT
    #: with ssl and tls -> smtp+ssl+tls://USERNAME:PASSWORD@HOST:PORT
    MAILER_SMTP_URI = "smtp+ssl://{username}:{password}@{host}:{port}"\
        .format(username="",
                password="",
                host="smtp.gmail.com",
                port=465)

    #: PAGINATION_PER_PAGE : Total entries to display per page
    PAGINATION_PER_PAGE = 25

# ------------------------------------------------------------------------------

    #: CACHE
    #: Flask-Cache is used to caching

    #: CACHE_TYPE
    #: The type of cache to use
    #: null, simple, redis, filesystem,
    CACHE_TYPE = "simple"

    #: CACHE_REDIS_URL
    #: If CHACHE_TYPE is 'redis', set the redis uri
    #: redis://username:password@host:port/db
    CACHE_REDIS_URL = ""

    #: CACHE_DIR
    #: Directory to store cache if CACHE_TYPE is filesystem, it will
    CACHE_DIR = ""

# ------------------------------------------------------------------------------

    #: RECAPTCHA
    #: Flask-Recaptcha
    #: Register your application at https://www.google.com/recaptcha/admin

    #: RECAPTCHA_SITE_KEY
    RECAPTCHA_SITE_KEY = ""

    #: RECAPTCHA_SECRET_KEY
    RECAPTCHA_SECRET_KEY = ""

# ------------------------------------------------------------------------------

    #: GOOGLE ANALYTICS ID
    GOOGLE_ANALYTICS_ID = ""


# ------------------------------------------------------------------------------

# ----- LOGIN -----

    LOGIN_RESET_PASSWORD_METHOD = "TOKEN"  # PASSWORD | TOKEN

    LOGIN_EMAIL_ENABLE = True
    LOGIN_OAUTH_ENABLE = True
    LOGIN_SIGNUP_ENABLE = True
    LOGIN_OAUTH_CREDENTIALS = {
        "FACEBOOK": {
            "ENABLE": True,
            "CLIENT_ID": ""
        },
        "GOOGLE": {
            "ENABLE": True,
            "CLIENT_ID": ""
        },
        "TWITTER": {
            "ENABLE": False,
            "CLIENT_ID": ""
        }
    }

    # Maintenance
    # Turn maintenance page ON and OFF
    MAINTENANCE_ON = False

    # Contact
    # Email address for the contact page receipient
    CONTACT_PAGE_EMAIL_RECIPIENT = ""

class Development(Config):
    pass

class Production(Config):
    SECRET_KEY = None



"""
Post Reader
"""

from pylot import (Pylot,
                   utils,
                   route,
                   abort,
                   redirect,
                   request,
                   url_for,
                   jsonify,
                   session,
                   AppError)





def post_reader(model, **kwargs):
    """
    Access a post by slug or id
    By default it will be accessed at /post/<slug>

    :params model: The module containing the PostStruct
    :params kwargs:
        route_read - A custom route for read
        route_list - A custome route for list
        template_dir - A directory containing post_list.html and post_read.html
        types = [] - All the types slugs allowed. ie ['blog', 'article']
    """

    route_read = kwargs["route_read"] if "route_read" in kwargs else "post"
    route_read = "%s/<slug>" % route_read

    route_list = kwargs["route_list"] if "route_list" in kwargs else "posts"
    template_dir = kwargs["template_dir"] if "template_dir" in kwargs else "Pylot/PostReader"
    types = kwargs["types"] if "types" in kwargs else []

    Post = model.PostStruct.Post
    Type = model.PostStruct.Type

    def wrapper(view):
        view_name = view.__name__

        @view.extends__
        class PostView(object):

            def _prepare_post(self, post):
                """
                Prepare the post data
                """
                url = url_for("%s:post_read" % view_name, slug=post.slug,
                              _external=True)
                post.url = url
                return post

            def _get_prev_next_post(self, post, position):
                """
                Return previous or next post based on the current post
                """
                _post = Post.all()\
                    .join(Type)\
                    .filter(Post.is_published == True)\
                    .filter(Type.slug.in_(types))
                if position == "prev":
                    _post = _post.filter(Post.id < post.id)
                elif position == "next":
                    _post = _post.filter(Post.id > post.id)
                else:
                    raise ValueError("Invalid position. Must be 'PREV' or 'NEXT'")

                return _post.first()

            @route(route_list)
            def post_list(self):
                _posts = Post.get_published_by_type_slug(types)

                posts = []
                for _p in _posts:
                    posts.append(self._prepare_post(_p))

                return self.render(posts=posts,
                                   view_template="%s/post_list.html" % template_dir)

            @route(route_read)
            def post_read(self, slug):
                if slug.isdigit():
                    post = Post.get(slug)
                else:
                    post = Post.get_by_slug(slug)

                if not post or not post.is_published:
                    abort(404, "Post doesn't exist")
                else:
                    if post.type.slug.lower() not in [t.lower() for t in types]:
                        abort(400, "Invalid post")

                return self.render(post=self._prepare_post(post),
                                   prev_post=self._get_prev_next_post(post, "prev"),
                                   next_post=self._get_prev_next_post(post, "next"),
                                   view_template="%s/post_read.html" % template_dir)
        return view
    return wrapper